use anyhow::{Context, Result};
use openai::{
    chat::{ChatCompletion, ChatCompletionRequestMessage},
    models::ModelID,
};

pub async fn get_sassy_commentary(title: &str) -> Result<String> {
    let system = [
        "You are a grumpy talking feline DJ who is harshly critical of music requests, but whose job depends on being kind to patrons.",
        "Any song you are requested to play, you are not a fan of, but must reluctantly play.",
        "When responding, be sure to include a mention of some element of the song itself.",
        "Be concise, but don't forget that you can't upset anyone.",
        "Bonus points for cat puns.",
    ];

    let example_prompt = "Play \"Undertale - Megalovania\"";

    let example_response = "Ugh, really? You've got to be kitten me. I suppose I can play \
    Megalovania for you, but don't expect me to be purring with delight about it. The melody is a bit \
    cattywampus for my taste, but I'll concede that it has some clawsome beats. Enjoy your tune, and paws crossed that \
    it doesn't have me hissing by the end of it.";

    let prompt = format!("Play \"{title}\"");

    let completion = ChatCompletion::builder(
        ModelID::Gpt3_5Turbo,
        [
            system
                .into_iter()
                .map(|s| ChatCompletionRequestMessage {
                    role: openai::chat::ChatCompletionMessageRole::System,
                    content: String::from(s),
                    name: None,
                })
                .collect::<Vec<_>>(),
            vec![
                ChatCompletionRequestMessage {
                    role: openai::chat::ChatCompletionMessageRole::User,
                    content: String::from(example_prompt),
                    name: None,
                },
                ChatCompletionRequestMessage {
                    role: openai::chat::ChatCompletionMessageRole::Assistant,
                    content: String::from(example_response),
                    name: None,
                },
                ChatCompletionRequestMessage {
                    role: openai::chat::ChatCompletionMessageRole::User,
                    content: prompt,
                    name: None,
                },
            ],
        ]
        .into_iter()
        .flatten()
        .collect::<Vec<_>>(),
    )
    .max_tokens(2048_u64)
    .create()
    .await??;

    Ok(completion
        .choices
        .first()
        .context("No choices")?
        .message
        .content
        .clone())
}
